@@ -1,15 +1,14 @@
 """
 Name server control tool.
 
-Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
+Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 import sys
 from Pyro4 import naming, errors
 
-if sys.version_info<(3, 0):
-    input=raw_input
+if sys.version_info < (3, 0):
+    input = raw_input
 
 
 def handleCommand(nameserver, options, args):
@@ -24,38 +23,38 @@ def handleCommand(nameserver, options, args):
         print("Name server ping ok.")
 
     def cmd_listprefix():
-        if len(args)==1:
+        if len(args) == 1:
             printListResult(nameserver.list())
         else:
             printListResult(nameserver.list(prefix=args[1]), "- prefix '%s'" % args[1])
 
     def cmd_listregex():
-        if len(args)!=2:
+        if len(args) != 2:
             raise SystemExit("requires one argument: pattern")
         printListResult(nameserver.list(regex=args[1]), "- regex '%s'" % args[1])
 
     def cmd_register():
-        if len(args)!=3:
+        if len(args) != 3:
             raise SystemExit("requires two arguments: name uri")
         nameserver.register(args[1], args[2], safe=True)
         print("Registered %s" % args[1])
 
     def cmd_remove():
-        count=nameserver.remove(args[1])
-        if count>0:
+        count = nameserver.remove(args[1])
+        if count > 0:
             print("Removed %s" % args[1])
         else:
             print("Nothing removed")
 
     def cmd_removeregex():
-        if len(args)!=2:
+        if len(args) != 2:
             raise SystemExit("requires one argument: pattern")
-        sure=input("Potentially removing lots of items from the Name server. Are you sure (y/n)?").strip()
+        sure = input("Potentially removing lots of items from the Name server. Are you sure (y/n)?").strip()
         if sure in ('y', 'Y'):
-            count=nameserver.remove(regex=args[1])
+            count = nameserver.remove(regex=args[1])
             print("%d items removed." % count)
 
-    commands={
+    commands = {
         "ping": cmd_ping,
         "list": cmd_listprefix,
         "listmatching": cmd_listregex,
@@ -66,11 +65,11 @@ def handleCommand(nameserver, options, args):
     try:
         commands[args[0]]()
     except Exception:
-        xt,xv,tb=sys.exc_info()
-        print("Error: %s - %s" % (xt.__name__,xv))
+        xt, xv, tb = sys.exc_info()
+        print("Error: %s - %s" % (xt.__name__, xv))
 
 
-def main(args):
+def main(args=None):
     from optparse import OptionParser
     usage = "usage: %prog [options] command [arguments]\nCommand is one of: " \
             "register remove removematching list listmatching ping"
@@ -78,7 +77,7 @@ def main(args):
     parser.add_option("-n", "--host", dest="host", help="hostname of the NS")
     parser.add_option("-p", "--port", dest="port", type="int",
                       help="port of the NS (or bc-port if host isn't specified)")
-    parser.add_option("-u","--unixsocket", help="Unix domain socket name of the NS")
+    parser.add_option("-u", "--unixsocket", help="Unix domain socket name of the NS")
     parser.add_option("-v", "--verbose", action="store_true", dest="verbose", help="verbose output")
     options, args = parser.parse_args(args)
     if not args or args[0] not in ("register", "remove", "removematching", "list", "listmatching", "ping"):
@@ -86,12 +85,11 @@ def main(args):
     if options.verbose:
         print("Locating name server...")
     if options.unixsocket:
-        options.host="./u:"+options.unixsocket
+        options.host = "./u:" + options.unixsocket
     try:
-
-        nameserver=naming.locateNS(options.host, options.port)
+        nameserver = naming.locateNS(options.host, options.port)
     except errors.PyroError:
-        x=sys.exc_info()[1]
+        x = sys.exc_info()[1]
         print("Failed to locate the name server: %s" % x)
         return
     if options.verbose:
@@ -100,5 +98,6 @@ def main(args):
     if options.verbose:
         print("Done.")
 
-if __name__=="__main__":
-    main(sys.argv[1:])
+
+if __name__ == "__main__":
+    main()
