@@ -4,17 +4,21 @@ Miscellaneous utilities.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
-import sys, zlib, logging
-import traceback, linecache
+import sys
+import zlib
+import logging
+import linecache
+import traceback
+import inspect
+import Pyro4.errors
+import Pyro4.message
+
 try:
     import copyreg
 except ImportError:
     import copy_reg as copyreg
-import Pyro4
-import Pyro4.errors
-import Pyro4.message
 
-log=logging.getLogger("Pyro4.util")
+log = logging.getLogger("Pyro4.util")
 
 
 def getPyroTraceback(ex_type=None, ex_value=None, ex_tb=None):
@@ -22,29 +26,31 @@ def getPyroTraceback(ex_type=None, ex_value=None, ex_tb=None):
     Pyro exception. Any remote Pyro exception information is included.
     Traceback information is automatically obtained via ``sys.exc_info()`` if
     you do not supply the objects yourself."""
+
     def formatRemoteTraceback(remote_tb_lines):
-        result=[" +--- This exception occured remotely (Pyro) - Remote traceback:"]
+        result = [" +--- This exception occured remotely (Pyro) - Remote traceback:"]
         for line in remote_tb_lines:
             if line.endswith("\n"):
                 line = line[:-1]
             lines = line.split("\n")
-            for line in lines:
+            for line2 in lines:
                 result.append("\n | ")
-                result.append(line)
+                result.append(line2)
         result.append("\n +--- End of remote traceback\n")
         return result
+
     try:
         if ex_type is not None and ex_value is None and ex_tb is None:
             # possible old (3.x) call syntax where caller is only providing exception object
             if type(ex_type) is not type:
                 raise TypeError("invalid argument: ex_type should be an exception type, or just supply no arguments at all")
         if ex_type is None and ex_tb is None:
-            ex_type, ex_value, ex_tb=sys.exc_info()
+            ex_type, ex_value, ex_tb = sys.exc_info()
 
-        remote_tb=getattr(ex_value, "_pyroTraceback", None)
-        local_tb=formatTraceback(ex_type, ex_value, ex_tb, Pyro4.config.DETAILED_TRACEBACK)
+        remote_tb = getattr(ex_value, "_pyroTraceback", None)
+        local_tb = formatTraceback(ex_type, ex_value, ex_tb, Pyro4.config.DETAILED_TRACEBACK)
         if remote_tb:
-            remote_tb=formatRemoteTraceback(remote_tb)
+            remote_tb = formatRemoteTraceback(remote_tb)
             return local_tb + remote_tb
         else:
             # hmm. no remote tb info, return just the local tb.
@@ -64,8 +70,8 @@ def formatTraceback(ex_type=None, ex_value=None, ex_tb=None, detailed=False):
         if type(ex_type) is not type:
             raise TypeError("invalid argument: ex_type should be an exception type, or just supply no arguments at all")
     if ex_type is None and ex_tb is None:
-        ex_type, ex_value, ex_tb=sys.exc_info()
-    if detailed and sys.platform!="cli":    # detailed tracebacks don't work in ironpython (most of the local vars are omitted)
+        ex_type, ex_value, ex_tb = sys.exc_info()
+    if detailed and sys.platform != "cli":  # detailed tracebacks don't work in ironpython (most of the local vars are omitted)
         def makeStrValue(value):
             try:
                 return repr(value)
@@ -74,46 +80,47 @@ def formatTraceback(ex_type=None, ex_value=None, ex_tb=None, detailed=False):
                     return str(value)
                 except:
                     return "<ERROR>"
+
         try:
-            result=["-"*52+"\n"]
+            result = ["-" * 52 + "\n"]
             result.append(" EXCEPTION %s: %s\n" % (ex_type, ex_value))
             result.append(" Extended stacktrace follows (most recent call last)\n")
-            skipLocals=True  # don't print the locals of the very first stackframe
+            skipLocals = True  # don't print the locals of the very first stack frame
             while ex_tb:
-                frame=ex_tb.tb_frame
-                sourceFileName=frame.f_code.co_filename
+                frame = ex_tb.tb_frame
+                sourceFileName = frame.f_code.co_filename
                 if "self" in frame.f_locals:
-                    location="%s.%s" % (frame.f_locals["self"].__class__.__name__, frame.f_code.co_name)
+                    location = "%s.%s" % (frame.f_locals["self"].__class__.__name__, frame.f_code.co_name)
                 else:
-                    location=frame.f_code.co_name
-                result.append("-"*52+"\n")
+                    location = frame.f_code.co_name
+                result.append("-" * 52 + "\n")
                 result.append("File \"%s\", line %d, in %s\n" % (sourceFileName, ex_tb.tb_lineno, location))
                 result.append("Source code:\n")
-                result.append("    "+linecache.getline(sourceFileName, ex_tb.tb_lineno).strip()+"\n")
+                result.append("    " + linecache.getline(sourceFileName, ex_tb.tb_lineno).strip() + "\n")
                 if not skipLocals:
-                    names=set()
+                    names = set()
                     names.update(getattr(frame.f_code, "co_varnames", ()))
                     names.update(getattr(frame.f_code, "co_names", ()))
                     names.update(getattr(frame.f_code, "co_cellvars", ()))
                     names.update(getattr(frame.f_code, "co_freevars", ()))
                     result.append("Local values:\n")
-                    for name in sorted(names):
-                        if name in frame.f_locals:
-                            value=frame.f_locals[name]
-                            result.append("    %s = %s\n" % (name, makeStrValue(value)))
-                            if name=="self":
+                    for name2 in sorted(names):
+                        if name2 in frame.f_locals:
+                            value = frame.f_locals[name2]
+                            result.append("    %s = %s\n" % (name2, makeStrValue(value)))
+                            if name2 == "self":
                                 # print the local variables of the class instance
-                                for name, value in vars(value).items():
-                                    result.append("        self.%s = %s\n" % (name, makeStrValue(value)))
-                skipLocals=False
-                ex_tb=ex_tb.tb_next
-            result.append("-"*52+"\n")
+                                for name3, value in vars(value).items():
+                                    result.append("        self.%s = %s\n" % (name3, makeStrValue(value)))
+                skipLocals = False
+                ex_tb = ex_tb.tb_next
+            result.append("-" * 52 + "\n")
             result.append(" EXCEPTION %s: %s\n" % (ex_type, ex_value))
-            result.append("-"*52+"\n")
+            result.append("-" * 52 + "\n")
             return result
         except Exception:
-            return ["-"*52+"\nError building extended traceback!!! :\n",
-                    "".join(traceback.format_exception(*sys.exc_info())) + '-'*52 + '\n',
+            return ["-" * 52 + "\nError building extended traceback!!! :\n",
+                    "".join(traceback.format_exception(*sys.exc_info())) + '-' * 52 + '\n',
                     "Original Exception follows:\n",
                     "".join(traceback.format_exception(ex_type, ex_value, ex_tb))]
     else:
@@ -145,26 +152,26 @@ class SerializerBase(object):
     def serializeData(self, data, compress=False):
         """Serialize the given data object, try to compress if told so.
         Returns a tuple of the serialized data (bytes) and a bool indicating if it is compressed or not."""
-        data=self.dumps(data)
+        data = self.dumps(data)
         return self.__compressdata(data, compress)
 
     def deserializeData(self, data, compressed=False):
         """Deserializes the given data (bytes). Set compressed to True to decompress the data first."""
         if compressed:
-            data=zlib.decompress(data)
+            data = zlib.decompress(data)
         return self.loads(data)
 
     def serializeCall(self, obj, method, vargs, kwargs, compress=False):
         """Serialize the given method call parameters, try to compress if told so.
         Returns a tuple of the serialized data and a bool indicating if it is compressed or not."""
-        data=self.dumpsCall(obj, method, vargs, kwargs)
+        data = self.dumpsCall(obj, method, vargs, kwargs)
         return self.__compressdata(data, compress)
 
     def deserializeCall(self, data, compressed=False):
         """Deserializes the given call data back to (object, method, vargs, kwargs) tuple.
         Set compressed to True to decompress the data first."""
         if compressed:
-            data=zlib.decompress(data)
+            data = zlib.decompress(data)
         return self.loadsCall(data)
 
     def loads(self, data):
@@ -180,10 +187,10 @@ class SerializerBase(object):
         raise NotImplementedError("implement in subclass")
 
     def __compressdata(self, data, compress):
-        if not compress or len(data)<200:
+        if not compress or len(data) < 200:
             return data, False  # don't waste time compressing small messages
-        compressed=zlib.compress(data)
-        if len(compressed)<len(data):
+        compressed = zlib.compress(data)
+        if len(compressed) < len(data):
             return compressed, True
         return data, False
 
@@ -200,9 +207,11 @@ class SerializerBase(object):
             try:
                 get_serializer_by_id(SerpentSerializer.serializer_id)
                 import serpent
+
                 def serpent_converter(obj, serializer, stream, level):
                     d = converter(obj)
                     serializer.ser_builtins_dict(d, stream, level)
+
                 serpent.register_class(clazz, serpent_converter)
             except Pyro4.errors.ProtocolError:
                 pass
@@ -240,7 +249,7 @@ class SerializerBase(object):
             if isinstance(obj, clazz):
                 return cls.__custom_class_to_dict_registry[clazz](obj)
         if type(obj) in (set, dict, tuple, list):
-            raise Pyro4.errors.ProtocolError("couldn't serialize sequence " + str(obj.__class__) + ", one of its elements is unserializable")
+            raise ValueError("couldn't serialize sequence " + str(obj.__class__) + ", one of its elements is unserializable")
         if hasattr(obj, "_pyroDaemon"):
             obj._pyroDaemon = None
         if isinstance(obj, BaseException):
@@ -282,12 +291,11 @@ class SerializerBase(object):
         classname = data.get("__class__", "<unknown>")
         if isinstance(classname, bytes):
             classname = classname.decode("utf-8")
-
         if classname in cls.__custom_dict_to_class_registry:
             converter = cls.__custom_dict_to_class_registry[classname]
             return converter(classname, data)
         if "__" in classname:
-            raise Pyro4.errors.SecurityError("refused to deserialize types with double underscores in their name: "+classname)
+            raise Pyro4.errors.SecurityError("refused to deserialize types with double underscores in their name: " + classname)
         # because of efficiency reasons the constructors below are hardcoded here instead of added on a per-class basis to the dict-to-class registry
         if classname.startswith("Pyro4.core."):
             if classname == "Pyro4.core.URI":
@@ -338,7 +346,7 @@ class SerializerBase(object):
         for serializer in _serializers.values():
             if classname == serializer.__class__.__name__:
                 return serializer
-        raise Pyro4.errors.ProtocolError("unsupported serialized class: "+classname)
+        raise Pyro4.errors.ProtocolError("unsupported serialized class: " + classname)
 
     @staticmethod
     def make_exception(exceptiontype, data):
@@ -368,11 +376,12 @@ class SerializerBase(object):
 
     def __eq__(self, other):
         """this equality method is only to support the unit tests of this class"""
-        return isinstance(other, SerializerBase) and vars(self)==vars(other)
+        return isinstance(other, SerializerBase) and vars(self) == vars(other)
 
     def __ne__(self, other):
         return not self.__eq__(other)
-    __hash__=object.__hash__
+
+    __hash__ = object.__hash__
 
 
 class PickleSerializer(SerializerBase):
@@ -398,6 +407,7 @@ class PickleSerializer(SerializerBase):
     def register_type_replacement(cls, object_type, replacement_function):
         def copyreg_function(obj):
             return replacement_function(obj).__reduce__()
+
         try:
             copyreg.pickle(object_type, copyreg_function)
         except TypeError:
@@ -428,7 +438,7 @@ class MarshalSerializer(SerializerBase):
 
     @classmethod
     def register_type_replacement(cls, object_type, replacement_function):
-        pass    # marshal serializer doesn't support per-type hooks
+        pass  # marshal serializer doesn't support per-type hooks
 
 
 class SerpentSerializer(SerializerBase):
@@ -458,6 +468,7 @@ class SerpentSerializer(SerializerBase):
                 serpent_serializer.ser_default_class(replaced, outputstream, indentlevel)
             else:
                 serpent_serializer._serialize(replaced, outputstream, indentlevel)
+
         serpent.register_class(object_type, custom_serializer)
 
 
@@ -466,27 +477,33 @@ class JsonSerializer(SerializerBase):
     serializer_id = Pyro4.message.SERIALIZER_JSON
 
     __type_replacements = {}
+
     def dumpsCall(self, obj, method, vargs, kwargs):
         data = {"object": obj, "method": method, "params": vargs, "kwargs": kwargs}
         data = json.dumps(data, ensure_ascii=False, default=self.default)
         return data.encode("utf-8")
+
     def dumps(self, data):
         data = json.dumps(data, ensure_ascii=False, default=self.default)
         return data.encode("utf-8")
+
     def loadsCall(self, data):
         data = data.decode("utf-8")
         data = json.loads(data)
         vargs = self.recreate_classes(data["params"])
         kwargs = self.recreate_classes(data["kwargs"])
         return data["object"], data["method"], vargs, kwargs
+
     def loads(self, data):
         data = data.decode("utf-8")
         return self.recreate_classes(json.loads(data))
+
     def default(self, obj):
         replacer = self.__type_replacements.get(type(obj), None)
         if replacer:
             obj = replacer(obj)
         return self.class_to_dict(obj)
+
     @classmethod
     def register_type_replacement(cls, object_type, replacement_function):
         cls.__type_replacements[object_type] = replacement_function
@@ -495,12 +512,15 @@ class JsonSerializer(SerializerBase):
 """The various serializers that are supported"""
 _serializers = {}
 _serializers_by_id = {}
+
+
 def get_serializer(name):
     try:
         return _serializers[name]
     except KeyError:
         raise Pyro4.errors.ProtocolError("serializer '%s' is unknown or not available" % name)
 
+
 def get_serializer_by_id(sid):
     try:
         return _serializers_by_id[sid]
@@ -512,7 +532,7 @@ try:
     import cPickle as pickle
 except ImportError:
     import pickle
-assert Pyro4.config.PICKLE_PROTOCOL_VERSION>=2, "pickle protocol needs to be 2 or higher"
+assert Pyro4.config.PICKLE_PROTOCOL_VERSION >= 2, "pickle protocol needs to be 2 or higher"
 _ser = PickleSerializer()
 _serializers["pickle"] = _ser
 _serializers_by_id[_ser.serializer_id] = _ser
@@ -534,13 +554,13 @@ try:
     else:
         ver = serpent.__version__
     ver = tuple(map(int, ver.split(".")))
-    if ver<(1, 5):
-        raise RuntimeError("requires serpent 1.5 or better")
+    if ver < (1, 7):
+        raise RuntimeError("requires serpent 1.7 or better")
     _ser = SerpentSerializer()
     _serializers["serpent"] = _ser
     _serializers_by_id[_ser.serializer_id] = _ser
 except ImportError:
-    # warnings.warn("serpent serializer not available", RuntimeWarning)
+    log.warning("serpent serializer is not available")
     pass
 del _ser
 
@@ -554,14 +574,18 @@ def resolveDottedAttribute(obj, attr, allowDotted):
     """
     if allowDotted:
         attrs = attr.split('.')
-        for i in attrs:
-            if i.startswith('_'):
-                raise AttributeError('attempt to access private attribute "%s"' % i)
+        for attr in attrs:
+            if attr.startswith('_'):
+                raise AttributeError("attempt to access private attribute '%s'" % attr)
             else:
-                obj = getattr(obj, i)
-        return obj
+                obj = getattr(obj, attr)
+    elif attr.startswith('_'):
+        raise AttributeError("attempt to access private attribute '%s'" % attr)
     else:
-        return getattr(obj, attr)
+        obj = getattr(obj, attr)
+    if not Pyro4.config.REQUIRE_EXPOSE or getattr(obj, "_pyroExposed", False):
+        return obj
+    raise AttributeError("attempt to access unexposed attribute '%s'" % attr)
 
 
 def excepthook(ex_type, ex_value, ex_tb):
@@ -591,3 +615,75 @@ def fixIronPythonExceptionForPickle(exceptionObject, addAttributes):
                     del piggyback["__ironpythonargs__"]
                     exceptionObject.args = exceptionObject.args[:-1]
                     exceptionObject.__dict__.update(piggyback)
+
+
+def get_exposed_members(obj, only_exposed=True):
+    """
+    Return public and exposed members of the given object's class.
+    You can also provide a class directly.
+    Private members are ignored no matter what (names starting with underscore).
+    If only_exposed is True, only members tagged with the @expose decorator are
+    returned. If it is False, all public members are returned.
+    The return value consists of the exposed methods, exposed attributes, and methods
+    tagged as @oneway.
+    (All this is used as meta data that Pyro sends to the proxy if it asks for it)
+    """
+    if not inspect.isclass(obj):
+        obj = obj.__class__
+    methods = set()  # all methods
+    oneway = set()  # oneway methods
+    attrs = set()  # attributes
+    for m in dir(obj):      # also lists names inherited from super classes
+        if m.startswith("_"):
+            continue
+        v = getattr(obj, m)
+        if inspect.ismethod(v) or inspect.isfunction(v):
+            if getattr(v, "_pyroExposed", not only_exposed):
+                methods.add(m)
+                # check if the method is marked with the @Pyro4.oneway decorator:
+                if getattr(v, "_pyroOneway", False):
+                    oneway.add(m)
+        elif inspect.isdatadescriptor(v):
+            func = v.fget or v.fset or v.fdel
+            assert func is not None
+            if getattr(func, "_pyroExposed", not only_exposed):
+                attrs.add(m)
+        # Note that we don't expose plain class attributes no matter what.
+        # it is a syntax error to add a decorator on them, and it is not possible
+        # to give them a _pyroExposed tag either.
+        # The way to expose attributes is by using properties for them.
+        # This automatically solves the protection/security issue: you have to
+        # explicitly decide to make an attribute into a @property (and to @expose it
+        # if REQUIRE_EXPOSED=True) before it is remotely accessible.
+    return {
+        "methods": methods,
+        "oneway": oneway,
+        "attrs": attrs
+    }
+
+
+def get_exposed_property_value(obj, propname, only_exposed=True):
+    """
+    Return the value of an @exposed @property.
+    If the requested property is not a @property or not exposed,
+    an AttributeError is raised instead.
+    """
+    v = getattr(obj.__class__, propname)
+    if inspect.isdatadescriptor(v):
+        if v.fget and getattr(v.fget, "_pyroExposed", not only_exposed):
+            return v.fget(obj)
+    raise AttributeError("attempt to access unexposed or unknown remote attribute '%s'" % propname)
+
+
+def set_exposed_property_value(obj, propname, value, only_exposed=True):
+    """
+    Sets the value of an @exposed @property.
+    If the requested property is not a @property or not exposed,
+    an AttributeError is raised instead.
+    """
+    v = getattr(obj.__class__, propname)
+    if inspect.isdatadescriptor(v):
+        pfunc = v.fget or v.fset or v.fdel
+        if v.fset and getattr(pfunc, "_pyroExposed", not only_exposed):
+            return v.fset(obj, value)
+    raise AttributeError("attempt to access unexposed or unknown remote attribute '%s'" % propname)
