@@ -10,6 +10,7 @@ import os
 import copy
 import pprint
 import pickle
+
 import Pyro4.util
 import Pyro4.errors
 import Pyro4.core
@@ -19,40 +20,42 @@ from testsupport import *
 
 
 class SerializeTests_pickle(unittest.TestCase):
-    SERIALIZER="pickle"
+    SERIALIZER = "pickle"
+
     def setUp(self):
-        self.previous_serializer=Pyro4.config.SERIALIZER
-        Pyro4.config.SERIALIZER=self.SERIALIZER
-        Pyro4.config.HMAC_KEY=b"testsuite"
-        self.ser=Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
+        self.previous_serializer = Pyro4.config.SERIALIZER
+        Pyro4.config.SERIALIZER = self.SERIALIZER
+        Pyro4.config.HMAC_KEY = b"testsuite"
+        self.ser = Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
+
     def tearDown(self):
-        Pyro4.config.HMAC_KEY=None
-        Pyro4.config.SERIALIZER=self.previous_serializer
-        
+        Pyro4.config.HMAC_KEY = None
+        Pyro4.config.SERIALIZER = self.previous_serializer
+
     def testSerItself(self):
-        s=Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
-        p,_=self.ser.serializeData(s)
-        s2=self.ser.deserializeData(p)
-        self.assertEqual(s,s2)
-        self.assertTrue(s==s2)
-        self.assertFalse(s!=s2)
+        s = Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
+        p, _ = self.ser.serializeData(s)
+        s2 = self.ser.deserializeData(p)
+        self.assertEqual(s, s2)
+        self.assertTrue(s == s2)
+        self.assertFalse(s != s2)
 
     def testSerUnicode(self):
         data = unicode("x")
-        ser,_ = self.ser.serializeData(data)
-        expected_type = str if sys.platform=="cli" else bytes   # ironpython serializes into str, not bytes... :(
+        ser, _ = self.ser.serializeData(data)
+        expected_type = str if sys.platform == "cli" else bytes  # ironpython serializes into str, not bytes... :(
         self.assertTrue(type(ser) is expected_type)
-        ser,_ = self.ser.serializeCall(data, unicode("method"), [], {})
+        ser, _ = self.ser.serializeCall(data, unicode("method"), [], {})
         self.assertTrue(type(ser) is expected_type)
 
     def testSerCompression(self):
-        d1,c1=self.ser.serializeData("small data", compress=True)
-        d2,c2=self.ser.serializeData("small data", compress=False)
+        d1, c1 = self.ser.serializeData("small data", compress=True)
+        d2, c2 = self.ser.serializeData("small data", compress=False)
         self.assertFalse(c1)
-        self.assertEqual(d1,d2)
-        bigdata="x"*1000
-        d1,c1=self.ser.serializeData(bigdata, compress=False)
-        d2,c2=self.ser.serializeData(bigdata, compress=True)
+        self.assertEqual(d1, d2)
+        bigdata = "x" * 1000
+        d1, c1 = self.ser.serializeData(bigdata, compress=False)
+        d2, c2 = self.ser.serializeData(bigdata, compress=True)
         self.assertFalse(c1)
         self.assertTrue(c2)
         self.assertTrue(len(d2) < len(d1))
@@ -60,80 +63,80 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual(bigdata, self.ser.deserializeData(d2, compressed=True))
 
     def testSerErrors(self):
-        e1=Pyro4.errors.NamingError(unicode("x"))
+        e1 = Pyro4.errors.NamingError(unicode("x"))
         e1._pyroTraceback = ["this is the remote traceback"]
         orig_e = copy.copy(e1)
-        e2=Pyro4.errors.PyroError(unicode("x"))
-        e3=Pyro4.errors.ProtocolError(unicode("x"))
-        if sys.platform=="cli":
+        e2 = Pyro4.errors.PyroError(unicode("x"))
+        e3 = Pyro4.errors.ProtocolError(unicode("x"))
+        if sys.platform == "cli":
             Pyro4.util.fixIronPythonExceptionForPickle(e1, True)
-        p,_=self.ser.serializeData(e1)
-        e=self.ser.deserializeData(p)
-        if sys.platform=="cli":
+        p, _ = self.ser.serializeData(e1)
+        e = self.ser.deserializeData(p)
+        if sys.platform == "cli":
             Pyro4.util.fixIronPythonExceptionForPickle(e, False)
-        self.assertTrue(isinstance(e, Pyro4.errors.NamingError))
+        self.assertIsInstance(e, Pyro4.errors.NamingError)
         self.assertEqual(repr(orig_e), repr(e))
         self.assertEqual(["this is the remote traceback"], e._pyroTraceback, "remote traceback info should be present")
-        p,_=self.ser.serializeData(e2)
-        e=self.ser.deserializeData(p)
-        self.assertTrue(isinstance(e, Pyro4.errors.PyroError))
+        p, _ = self.ser.serializeData(e2)
+        e = self.ser.deserializeData(p)
+        self.assertIsInstance(e, Pyro4.errors.PyroError)
         self.assertEqual(repr(e2), repr(e))
-        p,_=self.ser.serializeData(e3)
-        e=self.ser.deserializeData(p)
-        self.assertTrue(isinstance(e, Pyro4.errors.ProtocolError))
+        p, _ = self.ser.serializeData(e3)
+        e = self.ser.deserializeData(p)
+        self.assertIsInstance(e, Pyro4.errors.ProtocolError)
         self.assertEqual(repr(e3), repr(e))
-    
+
     def testSerializeExceptionWithAttr(self):
-        ex=ZeroDivisionError("test error")
-        ex._pyroTraceback=["test traceback payload"]
-        Pyro4.util.fixIronPythonExceptionForPickle(ex,True) # hack for ironpython
-        data,compressed=self.ser.serializeData(ex)
-        ex2=self.ser.deserializeData(data,compressed)
-        Pyro4.util.fixIronPythonExceptionForPickle(ex2,False) # hack for ironpython
+        ex = ZeroDivisionError("test error")
+        ex._pyroTraceback = ["test traceback payload"]
+        Pyro4.util.fixIronPythonExceptionForPickle(ex, True)  # hack for ironpython
+        data, compressed = self.ser.serializeData(ex)
+        ex2 = self.ser.deserializeData(data, compressed)
+        Pyro4.util.fixIronPythonExceptionForPickle(ex2, False)  # hack for ironpython
         self.assertEqual(ZeroDivisionError, type(ex2))
         self.assertTrue(hasattr(ex2, "_pyroTraceback"))
         self.assertEqual(["test traceback payload"], ex2._pyroTraceback)
 
     def testSerCoreOffline(self):
-        uri=Pyro4.core.URI("PYRO:9999@host.com:4444")
-        p,_=self.ser.serializeData(uri)
-        uri2=self.ser.deserializeData(p)
+        uri = Pyro4.core.URI("PYRO:9999@host.com:4444")
+        p, _ = self.ser.serializeData(uri)
+        uri2 = self.ser.deserializeData(p)
         self.assertEqual(uri, uri2)
-        self.assertEqual("PYRO",uri2.protocol)
-        self.assertEqual("9999",uri2.object)
-        self.assertEqual("host.com:4444",uri2.location)
+        self.assertEqual("PYRO", uri2.protocol)
+        self.assertEqual("9999", uri2.object)
+        self.assertEqual("host.com:4444", uri2.location)
         self.assertEqual(4444, uri2.port)
-        self.assertEqual(None, uri2.sockname)
+        self.assertIsNone(uri2.sockname)
 
-        uri=Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
-        p,_=self.ser.serializeData(uri)
-        uri2=self.ser.deserializeData(p)
+        uri = Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
+        p, _ = self.ser.serializeData(uri)
+        uri2 = self.ser.deserializeData(p)
         self.assertEqual(uri, uri2)
-        self.assertEqual("PYRO",uri2.protocol)
-        self.assertEqual("12345",uri2.object)
-        self.assertEqual("./u:/tmp/socketname",uri2.location)
-        self.assertEqual(None, uri2.port)
+        self.assertEqual("PYRO", uri2.protocol)
+        self.assertEqual("12345", uri2.object)
+        self.assertEqual("./u:/tmp/socketname", uri2.location)
+        self.assertIsNone(uri2.port)
         self.assertEqual("/tmp/socketname", uri2.sockname)
 
-        proxy=Pyro4.core.Proxy("PYRO:9999@host.com:4444")
-        proxy._pyroTimeout=42
-        self.assertTrue(proxy._pyroConnection is None)
-        p,_=self.ser.serializeData(proxy)
-        proxy2=self.ser.deserializeData(p)
-        self.assertTrue(proxy._pyroConnection is None)
-        self.assertTrue(proxy2._pyroConnection is None)
+        proxy = Pyro4.core.Proxy("PYRO:9999@host.com:4444")
+        proxy._pyroTimeout = 42
+        self.assertIsNone(proxy._pyroConnection)
+        p, _ = self.ser.serializeData(proxy)
+        proxy2 = self.ser.deserializeData(p)
+        self.assertIsNone(proxy._pyroConnection)
+        self.assertIsNone(proxy2._pyroConnection)
         self.assertEqual(proxy2._pyroUri, proxy._pyroUri)
         self.assertEqual(42, proxy2._pyroTimeout)
 
     def testNested(self):
-        if self.SERIALIZER=="marshal":
+        if self.SERIALIZER == "marshal":
             self.skipTest("marshal can't serialize custom objects")
-        uri1=Pyro4.core.URI("PYRO:1111@host.com:111")
-        uri2=Pyro4.core.URI("PYRO:2222@host.com:222")
-        _=self.ser.serializeData(uri1)
-        data=[uri1, uri2]
-        p,_=self.ser.serializeData(data)
-        [u1, u2]=self.ser.deserializeData(p)
+        uri1 = Pyro4.core.URI("PYRO:1111@host.com:111")
+        uri2 = Pyro4.core.URI("PYRO:2222@host.com:222")
+        _ = self.ser.serializeData(uri1)
+        data = [uri1, uri2]
+        p, _ = self.ser.serializeData(data)
+        [u1, u2] = self.ser.deserializeData(p)
         self.assertEqual(uri1, u1)
         self.assertEqual(uri2, u2)
 
@@ -142,22 +145,22 @@ class SerializeTests_pickle(unittest.TestCase):
         # but only to support serializing Pyro objects.
         # The serialized form of a Daemon should be empty (and thus, useless)
         with Pyro4.core.Daemon(port=0) as daemon:
-            d,_=self.ser.serializeData(daemon)
-            d2=self.ser.deserializeData(d)
-            self.assertTrue(len(d2.__dict__)==0, "deserialized daemon should be empty")
+            d, _ = self.ser.serializeData(daemon)
+            d2 = self.ser.deserializeData(d)
+            self.assertTrue(len(d2.__dict__) == 0, "deserialized daemon should be empty")
             try:
-                Pyro4.config.AUTOPROXY=False
-                obj=pprint.PrettyPrinter(stream="dummy", width=42)
-                obj.name="hello"
+                Pyro4.config.AUTOPROXY = False
+                obj = pprint.PrettyPrinter(stream="dummy", width=42)
+                obj.name = "hello"
                 daemon.register(obj)
-                o,_=self.ser.serializeData(obj)
-                if self.SERIALIZER=="pickle":
+                o, _ = self.ser.serializeData(obj)
+                if self.SERIALIZER == "pickle":
                     # only pickle can deserialize the PrettyPrinter class without the need of explicit deserialization function
-                    o2=self.ser.deserializeData(o)
+                    o2 = self.ser.deserializeData(o)
                     self.assertEqual("hello", o2.name)
                     self.assertEqual(42, o2._width)
             finally:
-                Pyro4.config.AUTOPROXY=True
+                Pyro4.config.AUTOPROXY = True
 
     def testPyroClasses(self):
         uri = Pyro4.core.URI("PYRO:object@host:4444")
@@ -165,38 +168,72 @@ class SerializeTests_pickle(unittest.TestCase):
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
-        uri=Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
+        uri = Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
         s, c = self.ser.serializeData(uri)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
-        proxy=Pyro4.core.Proxy(uri)
+        proxy = Pyro4.core.Proxy(uri)
+        proxy._pyroAttrs = set("abc")
+        proxy._pyroMethods = set("def")
+        proxy._pyroOneway = set("ghi")
+        proxy._pyroTimeout = 42
         s, c = self.ser.serializeData(proxy)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.core.Proxy)
         self.assertEqual(proxy._pyroUri, x._pyroUri)
-        self.assertTrue(x._)
-        daemon=Pyro4.core.Daemon()
+        self.assertEqual(42, x._pyroTimeout)
+        self.assertEqual(set("abc"), x._pyroAttrs)
+        self.assertEqual(set("def"), x._pyroMethods)
+        self.assertEqual(set("ghi"), x._pyroOneway)
+        daemon = Pyro4.core.Daemon()
         s, c = self.ser.serializeData(daemon)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.core.Daemon)
-        self.assertTrue(isinstance(x, Pyro4.core.Daemon))
         wrapper = Pyro4.futures._ExceptionWrapper(ZeroDivisionError("divided by zero"))
         s, c = self.ser.serializeData(wrapper)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.futures._ExceptionWrapper)
         self.assertEqual("divided by zero", str(x.exception))
 
+    def testPyroClassesForDict(self):
+        uri = Pyro4.core.URI("PYRO:object@host:4444")
+        state = uri.__getstate_for_dict__()
+        self.assertEqual(('PYRO', 'object', None, 'host', 4444), state)
+        uri2 = Pyro4.core.URI("PYRONAME:xxx")
+        uri2.__setstate_from_dict__(state)
+        self.assertEqual(uri, uri2)
+        proxy = Pyro4.core.Proxy(uri)
+        proxy._pyroAttrs = set("abc")
+        proxy._pyroMethods = set("def")
+        proxy._pyroOneway = set("ghi")
+        proxy._pyroTimeout = 42
+        state = proxy.__getstate_for_dict__()
+        self.assertEqual(('PYRO:object@host:4444', tuple(set("ghi")), tuple(set("def")), tuple(set("abc")), 42), state)
+        proxy2 = Pyro4.core.Proxy("PYRONAME:xxx")
+        proxy2.__setstate_from_dict__(state)
+        self.assertEqual(proxy, proxy2)
+        self.assertEqual(proxy._pyroUri, proxy2._pyroUri)
+        self.assertEqual(proxy._pyroAttrs, proxy2._pyroAttrs)
+        self.assertEqual(proxy._pyroMethods, proxy2._pyroMethods)
+        self.assertEqual(proxy._pyroOneway, proxy2._pyroOneway)
+        self.assertEqual(proxy._pyroTimeout, proxy2._pyroTimeout)
+        daemon = Pyro4.core.Daemon()
+        state = daemon.__getstate_for_dict__()
+        self.assertEqual(tuple(), state)
+        daemon2 = Pyro4.core.Daemon()
+        daemon2.__setstate_from_dict__(state)
+
     def testAutoProxy(self):
-        if self.SERIALIZER=="marshal":
+        if self.SERIALIZER == "marshal":
             self.skipTest("marshal can't serialize custom objects")
-        self.ser.register_type_replacement(MyThing2, Pyro4.core.pyroObjectToAutoProxy)
-        t1 = MyThing2("1")
-        t2 = MyThing2("2")
+        self.ser.register_type_replacement(MyThing, Pyro4.core.pyroObjectToAutoProxy)
+        t1 = MyThing("1")
+        t2 = MyThing("2")
         with Pyro4.core.Daemon() as d:
             d.register(t1, "thingy1")
             d.register(t2, "thingy2")
-            data = [t1, ["apple", t2] ]
+            data = [t1, ["apple", t2]]
             s, c = self.ser.serializeData(data)
             data = self.ser.deserializeData(s, c)
             self.assertEqual("apple", data[1][0])
@@ -206,27 +243,30 @@ class SerializeTests_pickle(unittest.TestCase):
             self.assertIsInstance(p2, Pyro4.core.Proxy)
             self.assertEqual("thingy1", p1._pyroUri.object)
             self.assertEqual("thingy2", p2._pyroUri.object)
+            self.assertEqual(set(["prop1", "prop2", "readonly_prop1"]), p1._pyroAttrs)
+            self.assertEqual(set(['classmethod', 'method', 'oneway', 'staticmethod', 'exposed']), p1._pyroMethods)
+            self.assertEqual(set(['oneway']), p1._pyroOneway)
 
     def testCustomClassFail(self):
-        if self.SERIALIZER=="pickle":
+        if self.SERIALIZER == "pickle":
             self.skipTest("pickle simply serializes custom classes")
         o = pprint.PrettyPrinter(stream="dummy", width=42)
         s, c = self.ser.serializeData(o)
         try:
-            x = self.ser.deserializeData(s, c)
+            _ = self.ser.deserializeData(s, c)
             self.fail("error expected, shouldn't deserialize unknown class")
         except Pyro4.errors.ProtocolError:
             pass
 
     def testCustomClassOk(self):
-        if self.SERIALIZER=="pickle":
+        if self.SERIALIZER == "pickle":
             self.skipTest("pickle simply serializes custom classes just fine")
-        o = MyThing2("test")
-        Pyro4.util.SerializerBase.register_class_to_dict(MyThing2, mything_dict)
+        o = MyThing("test")
+        Pyro4.util.SerializerBase.register_class_to_dict(MyThing, mything_dict)
         Pyro4.util.SerializerBase.register_dict_to_class("CUSTOM-Mythingymabob", mything_creator)
         s, c = self.ser.serializeData(o)
         o2 = self.ser.deserializeData(s, c)
-        self.assertIsInstance(o2, MyThing2)
+        self.assertIsInstance(o2, MyThing)
         self.assertEqual("test", o2.name)
         # unregister the deserializer
         Pyro4.util.SerializerBase.unregister_dict_to_class("CUSTOM-Mythingymabob")
@@ -236,25 +276,33 @@ class SerializeTests_pickle(unittest.TestCase):
         except Pyro4.errors.ProtocolError:
             pass  # ok
         # unregister the serializer
-        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing2)
+        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing)
         s, c = self.ser.serializeData(o)
         try:
             self.ser.deserializeData(s, c)
             self.fail("must fail")
         except Pyro4.errors.ProtocolError as x:
             msg = str(x)
-            self.assertTrue(msg in ["unsupported serialized class: testsupport.MyThing2",
-                                    "unsupported serialized class: PyroTests.testsupport.MyThing2"])
+            self.assertIn(msg, ["unsupported serialized class: testsupport.MyThing",
+                                "unsupported serialized class: PyroTests.testsupport.MyThing"])
 
     def testData(self):
         data = [42, "hello"]
         ser, compressed = self.ser.serializeData(data)
-        expected_type = str if sys.platform=="cli" else bytes   # ironpython serializes into str, not bytes... :(
+        expected_type = str if sys.platform == "cli" else bytes  # ironpython serializes into str, not bytes... :(
         self.assertTrue(type(ser) is expected_type)
         self.assertFalse(compressed)
         data2 = self.ser.deserializeData(ser, compressed=False)
         self.assertEqual(data, data2)
 
+    def testCircular(self):
+        data = [42, "hello", Pyro4.Proxy("PYRO:dummy@dummy:4444")]
+        data.append(data)
+        ser, compressed = self.ser.serializeData(data)
+        data2 = self.ser.deserializeData(ser, compressed)
+        self.assertIs(data2, data2[3])
+        self.assertEqual(42, data2[0])
+
     def testCallPlain(self):
         ser, compressed = self.ser.serializeCall("object", "method", "vargs", "kwargs")
         self.assertFalse(compressed)
@@ -265,7 +313,7 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual("kwargs", kwargs)
 
     def testCallPyroObjAsArg(self):
-        if self.SERIALIZER=="marshal":
+        if self.SERIALIZER == "marshal":
             self.skipTest("marshal can't serialize custom objects")
         uri = Pyro4.core.URI("PYRO:555@localhost:80")
         ser, compressed = self.ser.serializeCall("object", "method", [uri], {"thing": uri})
@@ -277,7 +325,7 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual({"thing": uri}, kwargs)
 
     def testCallCustomObjAsArg(self):
-        if self.SERIALIZER=="marshal":
+        if self.SERIALIZER == "marshal":
             self.skipTest("marshal can't serialize custom objects")
         e = ZeroDivisionError("hello")
         ser, compressed = self.ser.serializeCall("object", "method", [e], {"thing": e})
@@ -285,58 +333,58 @@ class SerializeTests_pickle(unittest.TestCase):
         obj, method, vargs, kwargs = self.ser.deserializeCall(ser, compressed=False)
         self.assertEqual("object", obj)
         self.assertEqual("method", method)
-        self.assertTrue(isinstance(vargs, list))
-        self.assertTrue(isinstance(vargs[0], ZeroDivisionError))
+        self.assertIsInstance(vargs, list)
+        self.assertIsInstance(vargs[0], ZeroDivisionError)
         self.assertEqual("hello", str(vargs[0]))
-        self.assertTrue(isinstance(kwargs["thing"], ZeroDivisionError))
+        self.assertIsInstance(kwargs["thing"], ZeroDivisionError)
         self.assertEqual("hello", str(kwargs["thing"]))
 
     def testSerializeException(self):
         e = ZeroDivisionError()
         d, c = self.ser.serializeData(e)
         e2 = self.ser.deserializeData(d, c)
-        self.assertTrue(isinstance(e2, ZeroDivisionError))
+        self.assertIsInstance(e2, ZeroDivisionError)
         self.assertEqual("", str(e2))
         e = ZeroDivisionError("hello")
         d, c = self.ser.serializeData(e)
         e2 = self.ser.deserializeData(d, c)
-        self.assertTrue(isinstance(e2, ZeroDivisionError))
+        self.assertIsInstance(e2, ZeroDivisionError)
         self.assertEqual("hello", str(e2))
         e = ZeroDivisionError("hello", 42)
         d, c = self.ser.serializeData(e)
         e2 = self.ser.deserializeData(d, c)
-        self.assertTrue(isinstance(e2, ZeroDivisionError))
-        self.assertTrue(str(e2) in ("('hello', 42)", "(u'hello', 42)"))
+        self.assertIsInstance(e2, ZeroDivisionError)
+        self.assertIn(str(e2), ("('hello', 42)", "(u'hello', 42)"))
         e.custom_attribute = 999
-        if sys.platform=="cli":
+        if sys.platform == "cli":
             Pyro4.util.fixIronPythonExceptionForPickle(e, True)
         ser, compressed = self.ser.serializeData(e)
         e2 = self.ser.deserializeData(ser, compressed)
-        if sys.platform=="cli":
+        if sys.platform == "cli":
             Pyro4.util.fixIronPythonExceptionForPickle(e2, False)
-        self.assertTrue(isinstance(e2, ZeroDivisionError))
-        self.assertTrue(str(e2) in ("('hello', 42)", "(u'hello', 42)"))
+        self.assertIsInstance(e2, ZeroDivisionError)
+        self.assertIn(str(e2), ("('hello', 42)", "(u'hello', 42)"))
         self.assertEqual(999, e2.custom_attribute)
 
     def testSerializeSpecialException(self):
-        self.assertTrue("GeneratorExit" in Pyro4.util.all_exceptions)
+        self.assertIn("GeneratorExit", Pyro4.util.all_exceptions)
         e = GeneratorExit()
         d, c = self.ser.serializeData(e)
         e2 = self.ser.deserializeData(d, c)
-        self.assertTrue(isinstance(e2, GeneratorExit))
+        self.assertIsInstance(e2, GeneratorExit)
 
     def testRecreateClasses(self):
-        self.assertEqual([1,2,3], self.ser.recreate_classes([1,2,3]))
-        d = {"__class__": "invalid" }
+        self.assertEqual([1, 2, 3], self.ser.recreate_classes([1, 2, 3]))
+        d = {"__class__": "invalid"}
         try:
             self.ser.recreate_classes(d)
             self.fail("error expected")
         except Pyro4.errors.ProtocolError:
             pass  # ok
-        d = {"__class__": "Pyro4.core.URI", "state": ['PYRO', '555', None, 'localhost', 80] }
+        d = {"__class__": "Pyro4.core.URI", "state": ['PYRO', '555', None, 'localhost', 80]}
         uri = self.ser.recreate_classes(d)
         self.assertEqual(Pyro4.core.URI("PYRO:555@localhost:80"), uri)
-        number, uri = self.ser.recreate_classes([1,{"uri": d}])
+        number, uri = self.ser.recreate_classes([1, {"uri": d}])
         self.assertEqual(1, number)
         self.assertEqual(Pyro4.core.URI("PYRO:555@localhost:80"), uri["uri"])
 
@@ -347,23 +395,38 @@ class SerializeTests_pickle(unittest.TestCase):
 
 class SerializeTests_serpent(SerializeTests_pickle):
     SERIALIZER = "serpent"
+
     @unittest.skip("pickle specific")
     def testProtocolVersion(self):
         pass
 
+    def testCircular(self):
+        with self.assertRaises(ValueError):  # serpent doesn't support object graphs (since serpent 1.7 reports ValueError instead of crashing)
+            super(SerializeTests_serpent, self).testCircular()
+
+
 class SerializeTests_json(SerializeTests_pickle):
     SERIALIZER = "json"
+
     @unittest.skip("pickle specific")
     def testProtocolVersion(self):
         pass
 
+    def testCircular(self):
+        with self.assertRaises(ValueError):  # json doesn't support object graphs
+            super(SerializeTests_json, self).testCircular()
 
-if os.name!="java":
+
+if os.name != "java":
     # The marshal serializer is not working correctly under jython,
     # see http://bugs.jython.org/issue2077
     # So we only include this when not running jython
     class SerializeTests_marshal(SerializeTests_pickle):
-        SERIALIZER="marshal"
+        SERIALIZER = "marshal"
+
+        def testCircular(self):
+            with self.assertRaises(ValueError):  # marshal doesn't support object graphs
+                super(SerializeTests_marshal, self).testCircular()
 
 
 class GenericTests(unittest.TestCase):
@@ -420,7 +483,7 @@ class GenericTests(unittest.TestCase):
         uri = Pyro4.core.URI("PYRO:object@host:4444")
         d = Pyro4.util.SerializerBase.class_to_dict(uri)
         self.assertEqual("Pyro4.core.URI", d["__class__"])
-        self.assertTrue("state" in d)
+        self.assertIn("state", d)
         x = Pyro4.util.SerializerBase.dict_to_class(d)
         self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
@@ -428,28 +491,28 @@ class GenericTests(unittest.TestCase):
         uri = Pyro4.core.URI("PYRO:12345@./u:/tmp/socketname")
         d = Pyro4.util.SerializerBase.class_to_dict(uri)
         self.assertEqual("Pyro4.core.URI", d["__class__"])
-        self.assertTrue("state" in d)
+        self.assertIn("state", d)
         x = Pyro4.util.SerializerBase.dict_to_class(d)
         self.assertIsInstance(x, Pyro4.core.URI)
         self.assertEqual(uri, x)
         self.assertEqual("/tmp/socketname", x.sockname)
 
     def testCustomDictClass(self):
-        o = MyThing2("test")
-        Pyro4.util.SerializerBase.register_class_to_dict(MyThing2, mything_dict)
+        o = MyThing("test")
+        Pyro4.util.SerializerBase.register_class_to_dict(MyThing, mything_dict)
         Pyro4.util.SerializerBase.register_dict_to_class("CUSTOM-Mythingymabob", mything_creator)
         d = Pyro4.util.SerializerBase.class_to_dict(o)
         self.assertEqual("CUSTOM-Mythingymabob", d["__class__"])
         self.assertEqual("test", d["name"])
         x = Pyro4.util.SerializerBase.dict_to_class(d)
-        self.assertIsInstance(x, MyThing2)
+        self.assertIsInstance(x, MyThing)
         self.assertEqual("test", x.name)
         # unregister the conversion functions and try again
-        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing2)
+        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing)
         Pyro4.util.SerializerBase.unregister_dict_to_class("CUSTOM-Mythingymabob")
         d_orig = Pyro4.util.SerializerBase.class_to_dict(o)
         clsname = d_orig["__class__"]
-        self.assertTrue(clsname.endswith("testsupport.MyThing2"))
+        self.assertTrue(clsname.endswith("testsupport.MyThing"))
         try:
             _ = Pyro4.util.SerializerBase.dict_to_class(d)
             self.fail("should crash")
@@ -495,9 +558,9 @@ def mything_dict(obj):
 def mything_creator(classname, d):
     assert classname == "CUSTOM-Mythingymabob"
     assert d["__class__"] == "CUSTOM-Mythingymabob"
-    return MyThing2(d["name"])
+    return MyThing(d["name"])
 
 
 if __name__ == "__main__":
-    #import sys;sys.argv = ['', 'Test.testName']
+    # import sys;sys.argv = ['', 'Test.testName']
     unittest.main()
